@@ -19,7 +19,7 @@
 #include "configuration.h"
 #include <string.h>
 #include <stdlib.h>
-#include "keyb.h"
+#include "event.h"
 
 static bool
 str2bool(char *s) {
@@ -55,7 +55,7 @@ void configuration::parse(FILE * in) {
             break;
           case CT_KEY:
             if (atoi(param) > 0)
-              key_redefine((ttkey)t->maxlen, (SDLKey)atoi(param));
+              events.set_keydef((event_t)t->maxlen,(SDLKey)atoi(param));
             break;
           default: assert_msg(0, "Unknown config data type.");
           }
@@ -67,7 +67,7 @@ void configuration::parse(FILE * in) {
   }
 }
 
-void configuration::register_entry(char *cnf_name, cnf_type  cnf_typ, void *cnf_var, long maxlen) {
+void configuration::register_entry(const char *cnf_name, cnf_type  cnf_typ, void *cnf_var, long maxlen) {
   config_data *t = new config_data;
 
   t->next = first_data;
@@ -121,11 +121,11 @@ configuration::configuration(FILE *glob, FILE *local) {
   CNF_INT(  "editor_pagesize",     &i_editor_towerpagesize );
   CNF_INT(  "editor_towerstarthei",&i_editor_towerstarthei );
   CNF_INT(  "waves_type",          &i_waves_type );
-  CNF_KEY(  "key_fire",             fire_key );
-  CNF_KEY(  "key_up",               up_key );
-  CNF_KEY(  "key_down",             down_key );
-  CNF_KEY(  "key_left",             left_key );
-  CNF_KEY(  "key_right",            right_key );
+  CNF_KEY(  "key_fire",             fire_ev );
+  CNF_KEY(  "key_up",               up_ev );
+  CNF_KEY(  "key_down",             down_ev );
+  CNF_KEY(  "key_left",             left_ev );
+  CNF_KEY(  "key_right",            right_ev );
   CNF_CHAR( "password",            &i_curr_password, PASSWORD_LEN );
   CNF_INT(  "start_lives",         &i_start_lives );
   CNF_INT(  "game_speed",          &i_game_speed);
@@ -170,7 +170,7 @@ configuration::~configuration(void) {
         fprintf(f, "%i", *(int *)t->cnf_var);
         break;
       case CT_KEY:
-        fprintf(f, "%i", (int)key_conv2sdlkey((ttkey)t->maxlen, true));
+        fprintf(f, "%i", (int)events.get_keydef_sdl((event_t)t->maxlen));
         break;
       default: assert_msg(0, "Unknown config data type.");
       }
