@@ -65,7 +65,7 @@ void dcl_setdebuglevel(int level) {
 }
 
 
-void debugprintf(int lvl, char *fmt, ...) {
+void debugprintf(int lvl, const char *fmt, ...) {
     if (lvl <= current_debuglevel) {
         va_list args;
         va_start(args, fmt);
