@@ -169,7 +169,7 @@
 #define PASSWORD_CHARS "abcdefghijklmnopqrstuvwxyz0123456789"
 
 void dcl_setdebuglevel(int level);
-void debugprintf(int lvl, char *fmt, ...);
+void debugprintf(int lvl, const char *fmt, ...);
 
 /* waits around 1/18 of a second */
 void dcl_wait(void);
@@ -194,8 +194,6 @@ FILE *create_local_data_file(const char *name);
  */
 bool get_data_file_path(const char * fname, char * f, int len);
 
-/* Is the TT window active? */
-extern bool tt_has_focus;
 
 #define MENU_DCLSPEED 4
 #define DEFAULT_GAME_SPEED 0
