@@ -22,7 +22,7 @@
 #include "points.h"
 #include "decl.h"
 #include "screen.h"
-#include "keyb.h"
+#include "event.h"
 #include "menu.h"
 #include "level.h"
 #include "elevators.h"
@@ -44,11 +44,11 @@ typedef enum {
 
 void gam_init(void) {
   scr_init();
-  key_init();
+  events.init();
 }
 
 void gam_done(void) {
-  key_done();
+  events.done();
   scr_done();
 }
 
@@ -77,7 +77,7 @@ void gam_arrival(void) {
   //TTSound->startsound(SND_START);
   top_hide();
 
-  key_readkey();
+  events.update();
 
   do {
     scr_drawall(8, 0, lev_towertime(), svisible, subshape, substart, SF_NONE);
@@ -153,11 +153,11 @@ void gam_arrival(void) {
       break;
     }
     dcl_wait();
-  } while (!((b == 5) || key_keypressed(fire_key)));
+  } while (!((b == 5) || events.check_for(fire_ev)));
 
   /* wait until the key has been released otherwise the space key
    might already result in a snowball being thrown */
-  key_wait_for_none(NULL);
+  events.wait_nokey(0);
   svisible = false;
 }
 
@@ -175,8 +175,8 @@ void gam_pick_up(Uint8 anglepos, Uint16 time) {
   bool svisible = false;
   int subshape = 0;
   int substart = 0;
-
-  key_readkey();
+  
+  events.update();
 
   do {
     scr_drawall(8, (4 - anglepos) & 0x7f, time, svisible, subshape, substart, SF_NONE);
@@ -232,7 +232,7 @@ void gam_pick_up(Uint8 anglepos, Uint16 time) {
       break;
     }
     dcl_wait();
-  } while (!((b == 6) | key_keypressed(fire_key)));
+  } while (!((b == 6) || events.check_for(fire_ev)));
 
   top_hide();
   svisible = false;
@@ -297,7 +297,7 @@ static void timeout(int &tower_position, int &tower_anglepos) {
   bg_tower_angle = tower_anglepos;
   bg_time = 0;
 
-  set_men_bgproc(game_background_proc);
+  Menusystem::set_bgproc(game_background_proc);
    
   men_info(_("Time over"), 150);
 }
@@ -361,7 +361,7 @@ static void bonus(int &tower_position, int &tower_angle, int time, bool lifes) {
   do {
      writebonus(tower_position, tower_angle, zeit, tec, extra, time, lif, lifes);
      dcl_wait();
-  } while ((delay++ < 80) && !key_keypressed(fire_key));
+  } while ((delay++ < 80) && !events.check_for(fire_ev));
 
   while (zeit > 0) {
     dcl_wait();
@@ -402,7 +402,7 @@ static void bonus(int &tower_position, int &tower_angle, int time, bool lifes) {
   do {
      writebonus(tower_position, tower_angle, zeit, tec, extra, time, lif, lifes);
      dcl_wait();
-  } while ((delay++ < 30) && (!key_keypressed(fire_key)));
+  } while ((delay++ < 30) && (!events.check_for(fire_ev)));
 }
 
 /* update the time */
@@ -420,13 +420,33 @@ static void akt_time(int &time, int &timecount, gam_states &state) {
   }
 }
 
-static void get_keys(Sint8 &left_right, Sint8 &up_down, bool &space, Uint16 kstat) {
+static void get_keys(Sint8 &left_right, Sint8 &up_down, bool &fire) {
 #ifdef GAME_DEBUG_KEYS
-  if ((kstat & left_key) && (kstat & right_key) &&
-      (kstat & up_key) &&   (kstat & down_key)) {
+    if(events.check_for(debug_ev)) {
       run_debug_menu();
-  }
+    }
 #endif /* GAME_DEBUG_KEYS */
+
+    if(events.check_for(left_ev)) {
+	left_right = -1;
+    } else if(events.check_for(right_ev)) {
+	left_right = 1;
+    } else {
+	left_right = 0;
+    }
+    
+    if(events.check_for(up_ev)) {
+	up_down = 1;
+    } else if(events.check_for(down_ev)) {
+	up_down = -1;
+    } else {
+	up_down = 0;
+    }
+
+    fire=events.check_for(fire_ev);
+}
+
+static void get_demo(Sint8 &left_right, Sint8 &up_down, bool &fire,Uint16 kstat) {
   if (kstat & left_key)
     left_right = -1;
   else {
@@ -446,9 +466,9 @@ static void get_keys(Sint8 &left_right, Sint8 &up_down, bool &space, Uint16 ksta
   }
 
   if (kstat & fire_key)
-    space = true;
+    fire = true;
   else
-    space = false;
+    fire = false;
 }
 
 static void escape(gam_states &state, int &tower_position, int &tower_anglepos, int time) {
@@ -459,7 +479,7 @@ static void escape(gam_states &state, int &tower_position, int &tower_anglepos,
   bg_tower_angle = tower_anglepos;
   bg_time = time;
 
-  set_men_bgproc(game_background_proc);
+  Menusystem::set_bgproc(game_background_proc);
   if (men_game())
     state = STATE_ABORTED;
 
@@ -468,7 +488,7 @@ static void escape(gam_states &state, int &tower_position, int &tower_anglepos,
            top_anglepos(), tower_anglepos);
 }
 
-static void pause(int &tower_position, int tower_anglepos, int time) {
+static void dopause(int &tower_position, int tower_anglepos, int time) {
 
   ttsounds::instance()->stopsound(SND_WATER);
 
@@ -476,7 +496,7 @@ static void pause(int &tower_position, int tower_anglepos, int time) {
   bg_tower_angle = tower_anglepos;
   bg_time = time;
    
-  set_men_bgproc(game_background_proc);
+  Menusystem::set_bgproc(game_background_proc);
   men_info(_("Pause"), -1, 1);
 
   ttsounds::instance()->startsound(SND_WATER);
@@ -525,7 +545,7 @@ gam_result gam_towergame(Uint8 &anglepos, Uint16 &resttime, int &demo, void *dem
   tower_angle = top_anglepos();
 
   ele_init();
-  key_readkey();
+  events.update();
 
   do {
 
@@ -535,11 +555,13 @@ gam_result gam_towergame(Uint8 &anglepos, Uint16 &resttime, int &demo, void *dem
 
     if ((demo > 0) && (demolen < demo) && dbuf) {
       demokeys = dbuf[demolen++];
-      get_keys(left_right, up_down, space, demokeys);
-      if ((demolen >= demo) || key_keystat()) state = STATE_ABORTED;
+      get_demo(left_right, up_down, space, demokeys);
+      Event *e=events.get_input();
+      if ((demolen >= demo) || e )
+	state = STATE_ABORTED;
+      if(e) free(e);
     } else {
-      demokeys = key_keystat();
-      get_keys(left_right, up_down, space, demokeys);
+      get_keys(left_right, up_down, space);
     }
 
     if (demo == -1) {
@@ -561,20 +583,19 @@ gam_result gam_towergame(Uint8 &anglepos, Uint16 &resttime, int &demo, void *dem
       break;
     }
 
-    if (key_keypressed(break_key)) {
+    if (events.check_for(break_ev)) {
       if (demo) state = STATE_ABORTED;
       else
         escape(state, tower_position, tower_angle, time);
     }
 
-    if (key_keypressed(pause_key)) {
+    if (events.check_for(pause_ev)) {
       if (demo) state = STATE_ABORTED;
       else
-        pause(tower_position, tower_angle, time);
+        dopause(tower_position, tower_angle, time);
     }
 
-    if (!demo) key_readkey();
-
+    events.update();
     ele_update();
     snb_movesnowball();
     top_updatetoppler(left_right, up_down, space);
@@ -646,7 +667,7 @@ gam_result gam_towergame(Uint8 &anglepos, Uint16 &resttime, int &demo, void *dem
 
   anglepos = top_anglepos();
   resttime = time;
-  key_readkey();
+  events.update();
 
   if (demo == -1) {
     demo = demolen;
