@@ -97,6 +97,7 @@ int main(int argc, char *argv[]) {
   DIR *dir = opendir("locale");
   bindtextdomain("toppler", dir == NULL ? LOCALEDIR : "locale");
   closedir(dir);
+  bind_textdomain_codeset("toppler", "UTF-8");
   textdomain("toppler");
 #endif
 
@@ -111,7 +112,6 @@ int main(int argc, char *argv[]) {
     SDL_WM_SetCaption(_("Nebulous"), NULL);
 
     int mouse = SDL_ShowCursor(config.fullscreen() ? 0 : 1);
-    tt_has_focus = true;
     atexit(QuitFunction);
     srand(time(0));
     startgame();
